
# q-expansion of newform 76.3.g.b, downloaded from the LMFDB on 13 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 76
weight = 3
poly_data = [100, 0, -10, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 0, 0], 1], [[0, 0, 1, 0], 10], [[0, 0, 0, 1], 10]]

hecke_ring_character_values = [[39, [-1, 0, 0, 0]], [21, [0, 0, -1, 0]]]
aps_data = [[2, 0, 0, 0], [-1, 1, -1, 0], [-2, -1, 2, 2], [4, 0, -8, 1], [-1, 0, 2, -5], [0, 0, -6, 0], [-16, -2, 16, 4], [6, 3, 9, -5], [4, -9, -2, 9], [0, 1, -2, 1], [-10, 0, 20, 7], [-24, 10, 0, -5], [13, -4, -13, 8], [-12, -10, -12, 0], [0, 7, 0, -7], [0, 4, 62, 4], [-13, -1, -13, 0], [0, -1, -22, -1], [-34, 21, 17, -21], [34, 14, 34, 0], [-13, 4, 13, -8], [-60, -8, -60, 0], [-19, 0, 38, -19], [0, -22, -8, -22], [-117, 2, 117, -4], [0, 27, -14, 27], [78, 0, -156, 10], [72, 0, -144, 18], [-16, 12, 16, -24], [-49, -8, 0, 4], [-68, 26, 34, -26], [47, -17, 47, 0], [0, 8, 155, 8], [146, 45, -73, -45], [56, 3, -56, -6], [-10, 0, 20, 7], [-80, -4, 80, 8], [5, 0, -10, 63], [-240, -28, 120, 28], [162, 12, -162, -24], [135, 0, -270, -9], [0, -27, 68, -27], [-120, 0, 240, 8], [112, 2, -112, -4], [-72, -30, 0, 15], [44, -46, -22, 46], [46, -68, 46, 0], [-30, -27, -30, 0], [-19, 0, 38, 19], [-208, 16, 0, -8], [55, -20, -55, 40], [48, 0, -96, -64], [0, 10, 121, 10], [-14, 55, 7, -55], [0, 56, 3, 56], [-32, -45, -32, 0], [-20, 32, 20, -64], [-84, 65, -84, 0], [142, 94, 0, -47], [0, -44, -217, -44], [79, 89, 79, 0], [46, -62, 0, 31], [-23, -41, -23, 0], [18, 0, -36, -119], [0, 100, 25, 100], [0, -48, -266, -48], [47, 0, -94, 7], [113, -70, -113, 140], [259, -35, 259, 0], [-332, 64, 0, -32], [-41, 20, 0, -10], [-38, 47, -38, 0], [-396, 13, 198, -13], [280, 8, 0, -4], [-178, 0, 356, 60], [168, -103, 168, 0], [0, 4, -400, 4], [184, -63, -184, 126], [-221, -40, 221, 80], [0, -6, 253, -6], [116, 0, -232, 200], [432, -69, -432, 138], [-324, -170, 162, 170], [0, 12, 308, 12], [-162, 3, -162, 0], [378, 35, -189, -35], [-513, -72, 0, 36], [-161, 176, 0, -88], [-306, -96, 306, 192], [-86, 0, 172, 26], [45, 0, -90, -79], [32, 16, -16, -16], [46, 0, -92, 21], [-184, 50, -184, 0], [-7, -187, -7, 0], [-268, -7, 134, 7], [0, -4, 120, -4], [-327, -60, 0, 30], [88, -170, -44, 170], [0, 16, -164, 16], [72, -86, -36, 86], [0, 72, -212, 72], [-179, 0, 358, 55], [536, -172, 0, 86], [-285, 0, 570, 19], [219, -88, 0, 44], [-548, -6, -548, 0], [0, -28, 283, -28], [12, 229, -6, -229], [493, 172, 0, -86], [-240, 0, 480, 35], [538, -84, -538, 168], [0, 6, 165, 6], [100, 0, -200, 82], [640, -41, -320, 41], [329, -12, -329, 24], [-261, 169, -261, 0], [-382, 0, 764, -162], [-256, 216, 0, -108], [872, 32, -436, -32], [0, -7, 270, -7], [176, 24, 0, -12], [-34, 320, 0, -160], [52, 0, -104, -120], [-92, 0, 184, 148], [-166, 191, 166, -382], [0, 181, 76, 181], [-42, -166, -42, 0], [-468, 180, -468, 0], [448, -90, 0, 45], [147, -175, 147, 0], [-54, -157, -54, 0], [-1028, 150, 514, -150], [-650, 84, 650, -168], [425, -308, 0, 154], [0, -104, 516, -104], [0, -139, 218, -139], [255, 0, -510, -169], [340, -342, 0, 171], [-213, 340, 0, -170], [-240, 266, 120, -266], [0, 108, 190, 108], [84, -66, -42, 66], [666, -19, -333, 19], [-600, -200, 0, 100], [256, -201, 256, 0], [570, 60, -570, -120], [-345, 40, 345, -80], [194, -237, -97, 237], [-626, 0, 1252, 195], [-790, 131, 790, -262], [-39, 88, 0, -44], [298, 193, -149, -193], [-288, -108, 144, 108], [-99, 337, -99, 0], [214, 0, -428, -241], [78, 0, -156, 10], [523, 132, -523, -264], [0, -248, -305, -248], [0, -32, -138, -32], [284, 180, 284, 0], [-1211, -16, 1211, 32], [-446, 130, -446, 0], [-231, -355, -231, 0], [-659, 236, 0, -118], [528, -24, 528, 0], [168, 448, 168, 0], [0, 177, -310, 177]]
